package org.springframework.orm.hibernate3;

import java.sql.SQLException;

import org.hibernate.FlushMode;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.support.OpenSessionInThread;
import org.springframework.util.Assert;

public class NotTransactionalHibernateTemplate extends HibernateTemplate {

    private OpenSessionInThread openSessionInThread;

    public NotTransactionalHibernateTemplate(SessionFactory sessionFactory, boolean allowCreate) {
	super(sessionFactory, allowCreate);
    }

    @Override
    public boolean isCheckWriteOperations() {
	return false;
    }

    /**
     * @param sessionFactory
     */
    public NotTransactionalHibernateTemplate(SessionFactory sessionFactory) {
	super(sessionFactory);
	openSessionInThread = new OpenSessionInThread(sessionFactory);
    }

    @Override
    public boolean isAllowCreate() {
	return false;
    }

    @Override
    public boolean isAlwaysUseNewSession() {
	return false;
    }

    @Override
    protected Session getSession() {
	Session session = openSessionInThread.getSession();
	session.setFlushMode(FlushMode.AUTO);
	return session;
    }

    @Override
    protected <T> T doExecute(HibernateCallback<T> action, boolean enforceNewSession, boolean enforceNativeSession) throws DataAccessException {

	Assert.notNull(action, "Callback object must not be null");
	Session session = (enforceNewSession ? SessionFactoryUtils.getNewSession(getSessionFactory(), getEntityInterceptor()) : getSession());
	boolean existingTransaction = (!enforceNewSession && (!isAllowCreate() || SessionFactoryUtils.isSessionTransactional(session, getSessionFactory())));
	if (existingTransaction) {
	    logger.debug("Found thread-bound Session for HibernateTemplate");
	}

	try {
	    Session sessionToExpose = (enforceNativeSession || isExposeNativeSession() ? session : createSessionProxy(session));
	    T result = action.doInHibernate(sessionToExpose);
	    session.flush();
	    return result;
	} catch (HibernateException ex) {
	    session.clear();
	    throw convertHibernateAccessException(ex);
	} catch (SQLException ex) {
	    session.clear();
	    throw convertJdbcAccessException(ex);
	} catch (RuntimeException ex) {
	    session.clear();
	    throw ex;
	}
    }

}
