/*
 * Copyright 2002-2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.orm.hibernate3.support;

import org.hibernate.FlushMode;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.dao.DataAccessResourceFailureException;
import org.springframework.orm.hibernate3.SessionFactoryUtils;
import org.springframework.orm.hibernate3.SessionHolder;
import org.springframework.transaction.support.TransactionSynchronizationManager;

public class OpenSessionInThread extends Thread {

    public static final String DEFAULT_SESSION_FACTORY_BEAN_NAME = "sessionFactory";

    private FlushMode flushMode = FlushMode.MANUAL;

    private SessionFactory sessionFactory;

    public OpenSessionInThread(SessionFactory sessionFactory) {
	super();
	this.sessionFactory = sessionFactory;
	Runtime.getRuntime().addShutdownHook(this);
    }

    public Session getSession() {

	Session session = null;
	if (TransactionSynchronizationManager.hasResource(sessionFactory)) {
	    session = ((SessionHolder) TransactionSynchronizationManager.getResource(sessionFactory)).getValidatedSession();
	} else {
	    session = getSession(sessionFactory);
	    TransactionSynchronizationManager.bindResource(sessionFactory, new SessionHolder(session));
	}

	return session;
    }

    protected Session getSession(SessionFactory sessionFactory) throws DataAccessResourceFailureException {
	Session session = SessionFactoryUtils.getSession(sessionFactory, true);
	FlushMode flushMode = this.flushMode;
	if (flushMode != null) {
	    session.setFlushMode(flushMode);
	}
	return session;
    }

    protected void closeSession() {
	Session session;
	if (this.sessionFactory != null) {
	    if (TransactionSynchronizationManager.hasResource(this.sessionFactory)) {
		session = ((SessionHolder) TransactionSynchronizationManager.getResource(this.sessionFactory)).getValidatedSession();
		SessionFactoryUtils.closeSession(session);
	    } else {
		System.err.println("can not close session, session is not in thread");
	    }
	} else {
	    System.err.println("can not close the session because session factory is null");
	}

    }

    @Override
    public void run() {
	closeSession();

    }
}
